<?php

namespace Drp\AdfsSocialiteProvider;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\ServiceProvider as BaseServiceProvider;
use Laravel\Socialite\Contracts\Factory;

class ServiceProvider extends BaseServiceProvider
{
    /**
     * Bootstrap services.
     */
    public function boot(): void
    {
        $manager = $this->app[Factory::class];
        $manager->extend('adfs', function ($app) {
            return new AdfsProvider(
                $app['request'],
                config('adfs.auth.client_id'),
                config('adfs.auth.client_secret'),
                route('adfs.callback')
            );
        });
    }

    /**
     * Register services.
     */
    public function register(): void
    {
        $this->publishes([
            __DIR__.'/database/migrations/2018_04_04_213239_add_adfs_id_to_users_table.php' => database_path('migrations/2018_04_04_213239_add_adfs_id_to_users_table.php'),
            __DIR__ . '/config/adfs.php' => config_path('adfs.php'),
        ]);
        $this->loadRoutesFrom(__DIR__.'/routes.php');
        $this->mergeConfigFrom(__DIR__ . '/config/adfs.php', 'adfs');

        $this->app->bind('adfs.redirect', function () {
           return redirect()->to('/');
        });

        $guard = config('auth.defaults.guard');
        $driver = config("auth.guards.$guard.driver");

        if ($driver === 'adfs') {
            Auth::viaRequest('adfs', function (Request $request) {
                $guard = Auth::createSessionDriver('web', [
                    'provider' => 'users',
                ]);

                $userId = $request->session()->get($guard->getName());

                if ($userId === null) {
                    return null;
                }

                $userId = Crypt::decrypt($userId);

                return TemporaryUser::fromId($userId);
            });
        }
    }
}
